package it.unimi.dsi.webgraph.algo;

/*		 
 * Copyright (C) 2010-2011 Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 3 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 */



import static org.junit.Assert.assertEquals;
import it.unimi.dsi.webgraph.ArrayListMutableGraph;
import it.unimi.dsi.webgraph.ImmutableGraph;

import java.util.BitSet;

import org.junit.Test;

public class StronglyConnectedComponentsTest {

	@Test
	public void testBuckets() {
		final ImmutableGraph g = new ArrayListMutableGraph( 9, 
				new int[][] { { 0, 0 }, { 1, 0 }, { 1, 2 }, 
				{ 2, 1 }, { 2, 3 }, { 2, 4 }, { 2, 5 },
				{ 3, 4 }, { 4, 3 },
				{ 5, 5 }, { 5, 6 }, { 5, 7 }, { 5, 8 },
				{ 6, 7 },
				{ 8, 7 } }
		).immutableView();

		StronglyConnectedComponents components = StronglyConnectedComponents.compute( g, true, null );

		BitSet buckets = new BitSet();
		buckets.set( 0 );
		buckets.set( 3 );
		buckets.set( 4 );
		assertEquals( buckets, components.buckets );
		assertEquals( 3, buckets.cardinality() );
		
		final int size[] = components.computeSizes();
		components.sortBySize( size );
		assertEquals( 2, size[ 0 ] );
		assertEquals( 2, size[ 1 ] );
		assertEquals( 1, size[ 2 ] );		
	}

	@Test
	public void testCompleteGraph() {
		StronglyConnectedComponents components = StronglyConnectedComponents.compute( ArrayListMutableGraph.newCompleteGraph( 5, false ).immutableView(),  true, null );
		assertEquals( 5, components.buckets.cardinality() );
		for( int i = 5; i-- != 0; ) assertEquals( 0, components.component[ i ] );
		assertEquals( 5, components.computeSizes()[ 0 ] );
	}

	@Test
	public void testTree() {
		StronglyConnectedComponents components = StronglyConnectedComponents.compute( ArrayListMutableGraph.newCompleteBinaryIntree( 3 ).immutableView(), true, null );
		assertEquals( 0, components.buckets.cardinality() );
		assertEquals( 15, components.numberOfComponents );
	}
}

